import unittest
import sys
import os
import inspect
from unittest_data_provider import data_provider

TESTS_PATH = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe()))) + '/../'
sys.path.insert(0, TESTS_PATH)

from infracheck.infracheck.runner import Runner
from infracheck.infracheck.config import ConfigLoader
from infracheck.infracheck.repository import Repository
from rkd.api.inputoutput import IO


class RunnerTest(unittest.TestCase):
    def provide_data():
        return [
            # url does not exist: will return False
            [
                'http',
                {
                    'url': 'https://duckduckgo-some-invalid-domain.com'
                },
                {
                    'on_each_up': ['echo "is up"'],
                    'on_each_down': ['echo "is down"']
                },
                False,
                'is down'
            ],

            # valid HTTP health check
            # checks also if the on_each_up hook is called
            [
                'http',
                {
                    'url': 'https://duckduckgo.com'
                },
                {
                    'on_each_up': ['echo "it works"'],
                    'on_each_down': ['echo "this should not show"']
                },
                True,
                'it works'
            ],
        ]

    @data_provider(provide_data)
    def test_run_with_hooks(self, check_type: str, input_data: dict, hooks: dict, expected_result: bool, expected_text: str):
        result = self._create_runner().run_single_check('some-check-name', check_type, input_data, hooks)

        self.assertEqual(expected_result, result.exit_status)
        self.assertEqual(expected_text, result.hooks_output)

    def test_injects_variables(self):
        out = self._create_runner().run_single_check(
            configured_name='some-check-name',
            check_name='printr',
            input_data={'message': 'Current user is ${ENV.USER}, running a ${checkName}'},
            hooks={}
        )

        self.assertEqual('Current user is ' + os.environ['USER'] + ', running a printr', out.output.strip())

    @staticmethod
    def _create_runner() -> Runner:
        dirs = [TESTS_PATH + '/../example/healthchecks', TESTS_PATH + '/infracheck/']

        return Runner(dirs,
                      config_loader=ConfigLoader(dirs),
                      repository=Repository(dirs),
                      io=IO())




import docker
import docker.errors
import time
from abc import abstractmethod


class BaseDockerContainerRequirement(object):
    docker_client: docker.DockerClient

    @classmethod
    def setUpClass(cls) -> None:
        cls.docker_client = docker.from_env()
        cls._remove_container()
        cls.docker_client.containers.run(cls._get_image_name(), name=cls._get_container_name(), ports=cls._get_ports(),
                                         detach=True, environment=cls._get_environment())
        cls._wait_for_container_to_be_ready()

    @classmethod
    def tearDownClass(cls) -> None:
        cls._remove_container()
        cls.docker_client.close()

    @classmethod
    def _remove_container(cls):
        try:
            container = cls.docker_client.containers.get(cls._get_container_name())

            try:
                container.kill()
            except docker.errors.APIError:
                pass

            container.remove()

        except docker.errors.NotFound:
            pass

    @classmethod
    def _wait_for_log_message(cls, message: str, timeout: int = 60):
        container = cls.docker_client.containers.get(cls._get_container_name())

        for sec in range(0, timeout):
            logs = container.logs().decode('utf-8')

            if message in logs:
                return True

            time.sleep(1)

        raise Exception('Message "{message}" not found in container logs'.format(message=message))

    @classmethod
    @abstractmethod
    def _wait_for_container_to_be_ready(cls):
        pass

    @staticmethod
    @abstractmethod
    def _get_container_name() -> str:
        pass

    @staticmethod
    @abstractmethod
    def _get_ports() -> dict:
        pass

    @staticmethod
    @abstractmethod
    def _get_image_name() -> str:
        pass

    @staticmethod
    @abstractmethod
    def _get_environment() -> dict:
        pass




import time
from .base_docker_container_test import BaseDockerContainerRequirement

DB = 'test'
USER = 'infracheck'
PASSWORD = 'solidarity_forever'


class InfluxDBContainerRequirement(BaseDockerContainerRequirement):
    @staticmethod
    def _get_environment() -> dict:
        return {
            'INFLUXDB_DB': DB,
            'INFLUXDB_ADMIN_USER': USER,
            'INFLUXDB_ADMIN_PASSWORD': PASSWORD
        }

    @classmethod
    def _wait_for_container_to_be_ready(cls):
        time.sleep(1)
        cls._wait_for_log_message('Listening on HTTP')
        time.sleep(1)

    @staticmethod
    def _get_container_name() -> str:
        return 'influxdb'

    @staticmethod
    def _get_ports() -> dict:
        return {'8086/tcp': '8086'}

    @staticmethod
    def _get_image_name() -> str:
        return 'influxdb:1.8-alpine'




import subprocess
import time
from .base_docker_container_test import BaseDockerContainerRequirement


class SSHServerContainerRequirement(BaseDockerContainerRequirement):
    @staticmethod
    def _wait_for_container_to_be_ready():
        out = ''

        for i in range(1, 600):
            try:
                out += str(subprocess.check_output('echo "ttttt\n\n" | nc -w 1 "localhost" "3222"',
                                                   shell=True, stderr=subprocess.STDOUT))

                if "SSH-2.0-OpenSSH" in out:
                    return
            except subprocess.CalledProcessError:
                time.sleep(0.5)

        raise Exception('SSH container did not get up properly. Output: ' + out)

    @staticmethod
    def get_current_ssh_server_fingerprint():
        return subprocess.check_output(
            'ssh-keyscan -t rsa -p 3222 localhost', stderr=subprocess.DEVNULL, shell=True
        ).decode('utf-8')

    @staticmethod
    def _get_container_name() -> str:
        return 'ssh'

    @staticmethod
    def _get_ports() -> dict:
        return {'22/tcp': 3222}

    @staticmethod
    def _get_image_name() -> str:
        return 'sickp/alpine-sshd:7.5'




import inspect
import sys
import os

path = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe()))) + '/../'
sys.path.insert(0, path)

from infracheck.infracheck.runner import Runner
from infracheck.infracheck.config import ConfigLoader
from infracheck.infracheck.repository import Repository
from infracheck.infracheck.model import ExecutedCheckResult
from rkd.api.inputoutput import IO


def run_check(check_type: str, input_data: dict, hooks: dict) -> ExecutedCheckResult:
    project_dirs = [path + '/../../example/healthchecks', path + '/../infracheck/']

    runner = Runner(project_dirs,
                    config_loader=ConfigLoader(project_dirs),
                    repository=Repository(project_dirs),
                    io=IO())

    return runner.run_single_check('example-check', check_type, input_data, hooks)



import unittest
import sys
import os
import inspect

TESTS_PATH = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe()))) + '/../'
sys.path.insert(0, TESTS_PATH)

from infracheck.infracheck.rkd_support import *


class RkdSupportTest(unittest.TestCase):
    def test_is_rkd_check(self):
        """
        Checks if RKD "url" has correct format
        """

        with self.subTest('Empty url'):
            self.assertFalse(is_rkd_check(''))

        with self.subTest('Empty url - only schema present'):
            self.assertFalse(is_rkd_check('rkd://'))

        with self.subTest('Url with only import name, without task name'):
            self.assertFalse(is_rkd_check('rkd://rkd.standardlib.shell'))

        with self.subTest('Fully complete url'):
            self.assertTrue(is_rkd_check('rkd://rkd.standardlib.shell:sh'))

    def test_split_rkd_path(self):
        """
        Splits URL into module and task names
        """

        self.assertEqual(
            ('rkd.standardlib.shell', ':test'),
            split_rkd_path('rkd://rkd.standardlib.shell:test')
        )

    def test_prepare_rkd_check_bin_path(self):
        self.assertIn(
            "'-m', 'rkd', '--imports', 'rkd.standardlib.shell', ':test'",
            str(prepare_rkd_check_bin_path('rkd://rkd.standardlib.shell:test'))
        )

    def test_rkd_module_exists(self):
        with self.subTest('Success case - module and task are found'):
            self.assertTrue(rkd_module_exists('rkd://rkd.standardlib.shell:sh'))

        with self.subTest('Failure case - module not found'):
            self.assertFalse(rkd_module_exists('rkd://rkd.standardlib.not_existing_name:sh'))

        with self.subTest('Failure case - task not found in module'):
            self.assertFalse(rkd_module_exists('rkd://rkd.standardlib.shell:non-existing-task'))

    def test_add_rkd_environment_variables(self):
        self.assertEqual({'RKD_UI': 'false'}, add_rkd_environment_variables({}, 'test'))




"""
HTTP
====

Exposes simple HTTP endpoint with JSON response
"""

import tornado.ioloop
import tornado.web
import json
from .controller import Controller
from .model import ExecutedChecksResultList


class CheckExposingHandler(tornado.web.RequestHandler):  # pragma: no cover
    app: Controller

    def get(self):
        result: ExecutedChecksResultList = self.app.retrieve_checks()

        self.set_status(500 if not result.is_global_status_success() else 200)
        self.add_header('Content-Type', 'application/json')
        self.write(
            json.dumps(result.to_hash(), sort_keys=True, indent=4, separators=(',', ': '))
        )

    def data_received(self, chunk):
        pass


class VersionHandler(tornado.web.RequestHandler):  # pragma: no cover
    app: Controller

    def get(self):
        self.set_status(200)
        self.add_header('Content-Type', 'application/json')
        self.write(
            json.dumps(self.app.get_version(), sort_keys=True, indent=4, separators=(',', ': '))
        )

    def data_received(self, chunk):
        pass


class HttpServer(object):
    app = None
    port: int
    path_prefix: str

    def __init__(self, app: Controller, port: int, server_path_prefix: str):
        self.app = app
        self.port = port
        self.path_prefix = server_path_prefix

    def run(self):
        CheckExposingHandler.app = self.app
        VersionHandler.app = self.app

        srv = tornado.web.Application([
            (r"" + self.path_prefix + "/", CheckExposingHandler),
            (r"" + self.path_prefix + "/version", VersionHandler)
        ])

        srv.listen(self.port)
        tornado.ioloop.IOLoop.current().start()



from typing import Dict


class InfracheckException(Exception):
    pass


class RunnerException(InfracheckException):
    @staticmethod
    def from_invalid_variable_error(var_name: str, check_name: str, available_vars: Dict[str, str]) \
            -> 'RunnerException':

        return RunnerException(
            'Invalid variable "%s" in check %s. Available variables: %s' %
            (var_name, check_name, str(available_vars.keys()))
        )

    @staticmethod
    def from_non_existing_executable(check_name: str) -> 'RunnerException':
        return RunnerException('Healthcheck executable "{check_name}" does not exist'.format(check_name=check_name))

    @staticmethod
    def from_expected_list_of_hooks(hook_name: str):
        return RunnerException('Expected a LIST of hooks in "{hook_name}"'.format(hook_name=hook_name))


class ConfigurationException(InfracheckException):
    @classmethod
    def from_rkd_module_not_existing(cls, check_name: str) -> 'ConfigurationException':
        return cls(
            'RKD module cannot be imported, or task name is invalid: {check_name}'.format(check_name=check_name)
        )

    @classmethod
    def from_binary_not_found(cls, check_name: str, paths: list) -> 'ConfigurationException':
        return cls(
            'Invalid check type "{check_name}", was looking in: "{paths}"'.format(
                check_name=check_name,
                paths=str(paths)
            )
        )

    @classmethod
    def from_rkd_check_url_error(cls, check_name):
        return cls(
            'RiotKit-Do check syntax "{}" is invalid. Valid example: rkd://rkd.standardlib.shell:sh'.format(check_name)
        )



import unittest
import docker
import os

from tempfile import NamedTemporaryFile
from tests.utils import run_check
from tests.utils.ssh_test import SSHServerContainerRequirement


class SshCommandCheckTest(SSHServerContainerRequirement, unittest.TestCase):
    docker_client: docker.DockerClient

    def test_fingerprint_will_be_fetched_first_time(self):
        known_hosts_file = NamedTemporaryFile(delete=False)

        result = run_check('ssh-command', {
            'HOST': 'localhost',
            'PORT': 3222,
            'USER': 'root',
            'PASSWORD': 'root',
            'KNOWN_HOSTS_FILE': known_hosts_file.name,
            'SSH_OPTS': '',  # there is no StrictHostKeyChecking turned off
            'COMMAND': 'ls -la'
        }, {})

        os.unlink(known_hosts_file.name)
        self.assertTrue(result.exit_status)

    def test_not_passed_host_raises_human_readable_message(self):
        result = run_check('ssh-command', {}, {})

        self.assertIn('HOST is mandatory', result.output.strip())
        self.assertFalse(result.exit_status)

    def test_success_case(self):
        """
        Simple success case with expected keywords usage
        :return:
        """

        result = run_check('ssh-command', {
            'HOST': 'localhost',
            'PORT': 3222,
            'USER': 'root',
            'PASSWORD': 'root',
            'SSH_OPTS': '-o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null',
            'COMMAND': 'uname -a',
            'EXPECTED_KEYWORDS': 'Linux',
            'UNEXPECTED_KEYWORDS': 'Darwin'
        }, {})

        self.assertEqual('OK', result.output.strip())
        self.assertTrue(result.exit_status)

    def test_invalid_password(self):
        result = run_check('ssh-command', {
            'HOST': 'localhost',
            'PORT': 3222,
            'USER': 'root',
            'PASSWORD': 'invalid-password',
            'SSH_OPTS': '-o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null'
        }, {})

        self.assertIn('Permission denied, please try again.', result.output.strip())
        self.assertFalse(result.exit_status)



from datetime import datetime
from typing import Dict


class ExecutedCheckResult(object):
    """
    Represents a single result of a single check
    """

    output: str
    exit_status: bool
    hooks_output: str
    configured_name: str
    refresh_time: datetime

    def __init__(self, configured_name: str, output: str, exit_status: bool, hooks_output: str):
        self.configured_name = configured_name
        self.output = output
        self.exit_status = exit_status
        self.hooks_output = hooks_output
        self.refresh_time = datetime.now()

    @classmethod
    def from_not_ready(cls, configured_name: str):
        check = cls(
            configured_name=configured_name,
            output='Check not ready',
            exit_status=False,
            hooks_output=''
        )

        check.refresh_time = None

        return check

    def to_hash(self) -> dict:
        return {
            'status': self.exit_status,
            'output': self.output,
            'hooks_output': self.hooks_output,
            'ident': self.configured_name + '=' + str(self.exit_status),
            'checked_at': self.refresh_time.strftime('%Y-%m-%d %H-%M-%S') if self.refresh_time else ''
        }


class ExecutedChecksResultList(object):
    checks: Dict[str, ExecutedCheckResult]

    def __init__(self):
        self.checks = {}

    def add(self, config_name: str, result: ExecutedCheckResult) -> None:
        self.checks[config_name] = result

    def to_hash(self) -> dict:
        checks_as_hash = {}

        for name, details in self.checks.items():
            checks_as_hash[name] = details.to_hash()

        return {
            'checks': checks_as_hash,
            'global_status': self.is_global_status_success()
        }

    def is_global_status_success(self) -> bool:
        for name, details in self.checks.items():
            if not details.exit_status:
                return False

        return True




"""
Runner
======

Executes checks and captures result
"""

import subprocess
import os
import json
import re
import time
from datetime import datetime
from rkd.api.inputoutput import IO
from .exceptions import RunnerException
from .model import ExecutedCheckResult, ExecutedChecksResultList
from .repository import Repository
from .config import ConfigLoader
from .rkd_support import is_rkd_check, prepare_rkd_check_bin_path, add_rkd_environment_variables


class Runner(object):
    paths: list
    timeout: int
    wait_time: int
    config_loader: ConfigLoader
    repository: Repository
    io: IO

    def __init__(self, dirs: list, config_loader: ConfigLoader, repository: Repository, io: IO,
                 timeout: int = 1800, wait_time: int = 0):

        self.timeout = timeout
        self.wait_time = wait_time
        self.paths = []
        self.config_loader = config_loader
        self.repository = repository
        self.io = io

        for path in dirs:
            self.paths.append(path + '/checks')

    def run_single_check(self, configured_name: str, check_name: str, input_data: dict, hooks: dict) \
            -> ExecutedCheckResult:

        """
        Runs a single check and returns result

        :param configured_name:
        :param check_name:
        :param input_data:
        :param hooks:
        :return:
        """

        self.io.debug('Executing check {}'.format(configured_name))
        bin_path = self._get_check_path(check_name)
        bin_path = self._append_commandline_switches(input_data, bin_path)

        try:
            self.io.debug('bin_path=' + str(bin_path))

            env = {**dict(os.environ), **self._prepare_data(check_name, input_data)}
            env = self._add_environment_variables(env, check_name)

            timeout = env['INFRACHECK_TIMEOUT'] if 'INFRACHECK_TIMEOUT' in env else self.timeout

            output = subprocess.check_output(bin_path, env=env, stderr=subprocess.PIPE, timeout=timeout)
            exit_status = True

        except subprocess.CalledProcessError as e:
            output = e.output + e.stderr
            self.io.warn('{} returned error: {}'.format(configured_name, output.decode('utf-8')))
            exit_status = False

        except subprocess.TimeoutExpired as e:
            output = b'Timed out: '

            if e.output:
                output += e.output

            if e.stderr:
                output += e.stderr

            self.io.error('{} timed out and returned: {}'.format(configured_name, output.decode('utf-8')))
            exit_status = False

        self.io.debug('Execution finished, running hooks...')
        hooks_out = self._notify_hooks(hooks, exit_status, self.io)

        return ExecutedCheckResult(
            output=output.decode('utf-8'),
            exit_status=exit_status,
            hooks_output=hooks_out,
            configured_name=configured_name
        )

    def run_checks(self, enabled_configs: list) -> None:
        """
        Runs checks one-by-one and saves to cache

        :param enabled_configs: List of enabled configuration files (json files)
        :return:
        """

        for config_name in enabled_configs:
            result = None
            config = self.config_loader.load(config_name)

            if not result:
                result = self.run_single_check(config_name, config['type'], config['input'], config.get('hooks', {}))
                self.repository.push_to_cache(config_name, result)

            if self.wait_time > 0:
                time.sleep(self.wait_time)

    def get_checks_results(self, enabled_configs: list) -> ExecutedChecksResultList:
        """
        Get results previously generated by runner

        :param enabled_configs:
        :return:
        """

        results = ExecutedChecksResultList()

        for config_name in enabled_configs:
            result = self.repository.retrieve_from_cache(config_name)

            if not result:
                result = ExecutedCheckResult.from_not_ready(configured_name=config_name)

            results.add(config_name, result)

        return results

    @staticmethod
    def _add_environment_variables(env: dict, check_name: str):
        if is_rkd_check(check_name):
            env = add_rkd_environment_variables(env, check_name)

        return env

    @staticmethod
    def _append_commandline_switches(input_data: dict, bin_path: list) -> list:
        """
        Inject commandline switches

        :param input_data:
        :param bin_path:
        :return:
        """

        for name, value in input_data.items():
            name: str

            if name.startswith('--'):
                bin_path.append(name + '=' + str(value))

            elif name.startswith('-'):
                bin_path.append(name)
                bin_path.append(str(value))

        return bin_path

    @staticmethod
    def _prepare_data(check_name: str, input_data: dict):
        """ Serialize and parse """

        output_data = {}

        for key, value in input_data.items():
            key: str

            if type(value) == dict or type(value) == list:
                value = json.dumps(value)

            if key.startswith('-'):
                continue

            output_data[key.upper()] = Runner._inject_variables(check_name, str(value))

        return output_data

    @staticmethod
    def _inject_variables(check_name: str, value: str) -> str:
        """
        Inject variables, including environment variables from host,
        to allow for example secure passing of passwords
        """

        matches = re.findall(r'\${([A-Za-z0-9_.]+)\}', value)

        if not matches:
            return value

        variables = {
            'checkName': check_name,
            'date': datetime.now().isoformat(),
            'timestamp': str(datetime.now().timestamp())
        }

        for env_name, env_value in os.environ.items():
            variables['ENV.' + env_name.upper()] = env_value

        for match in matches:
            if match not in variables:
                raise RunnerException.from_invalid_variable_error(match, check_name, variables)

            value = value.replace('${%s}' % match, variables[match])

        return value

    def _get_check_path(self, check_name: str) -> list:
        if is_rkd_check(check_name):
            return prepare_rkd_check_bin_path(check_name)

        for path in self.paths:
            if os.path.isfile(path + '/' + check_name):
                return [path + '/' + check_name]

        raise RunnerException.from_non_existing_executable(check_name)

    @staticmethod
    def _notify_hooks(hooks: dict, exit_status: bool, io: IO) -> str:
        mapping = {
            True: 'on_each_up',
            False: 'on_each_down'
        }

        out = ""

        if exit_status in mapping and mapping[exit_status] in hooks:
            commands = hooks[mapping[exit_status]]

            if type(commands).__name__ != 'list':
                raise RunnerException.from_expected_list_of_hooks(mapping[exit_status])

            for command in commands:
                io.debug('Triggering hook command "{}"'.format(command))

                try:
                    out += subprocess.check_output(command, shell=True, timeout=1800).decode('utf-8').strip()

                except subprocess.CalledProcessError as e:
                    io.error('Cannot execute hook command "{cmd}". Error: {err}'.format(
                        cmd=command, err=str(e.output) + str(e.stderr))
                    )
                except subprocess.TimeoutExpired:
                    io.error('Cannot execute hook command "{cmd}. Timed out while executing command"'.format(
                        cmd=command)
                    )
                except Exception:
                    io.error('Cannot execute hook command "{cmd}. Unknown error"'.format(cmd=command))

        return out



import unittest
import docker

from tests.utils import run_check
from tests.utils.ssh_test import SSHServerContainerRequirement


class SshFingerprintTest(SSHServerContainerRequirement, unittest.TestCase):
    docker_client: docker.DockerClient

    def test_success_case(self):
        # we create SSH server for testing dynamically in a docker container, so each time it has a different identity
        current_expected_fingerprint = self.get_current_ssh_server_fingerprint()

        result = run_check('ssh-fingerprint', {
            'HOST': 'localhost',
            'PORT': 3222,
            'EXPECTED_FINGERPRINT': current_expected_fingerprint
        }, {})

        self.assertEqual('Fingerprint is OK', result.output.strip())
        self.assertTrue(result.exit_status)

    def test_invalid_fingerprint(self):
        result = run_check('ssh-fingerprint', {
            'HOST': 'localhost',
            'PORT': 3222,
            'EXPECTED_FINGERPRINT': 'SOME FINGERPRINT THAT DOES NOT MATCH SERVER FINGERPRINT'
        }, {})

        self.assertIn('Fingerprint does not match', result.output.strip())
        self.assertFalse(result.exit_status)

    def test_missing_host_parameter(self):
        result = run_check('ssh-fingerprint', {
            'PORT': 3222,
            'EXPECTED_FINGERPRINT': 'SOME FINGERPRINT THAT DOES NOT MATCH SERVER FINGERPRINT'
        }, {})

        self.assertIn('You need to provide a HOST', result.output.strip())
        self.assertFalse(result.exit_status)



"""
RKD Support
===========

Methods used to add RiotKit-Do support in the InfraCheck
"""

import sys
from typing import Tuple

from rkd.exception import ParsingException
from rkd.api.contract import TaskDeclarationInterface
from rkd.api.parsing import SyntaxParsing
from .exceptions import ConfigurationException


def is_rkd_check(check_name: str) -> bool:
    schema_length = len('rkd://')

    if not check_name[0:schema_length] == 'rkd://':
        return False

    return ":" in check_name[schema_length:]


def split_rkd_path(check_name: str) -> Tuple[str, str]:
    if not is_rkd_check(check_name):
        raise ConfigurationException.from_rkd_check_url_error(check_name)

    module_name, task_name = check_name[len('rkd://'):].split(':', 1)

    return module_name, ':' + task_name


def prepare_rkd_check_bin_path(check_name: str) -> list:
    module_name, task_name = split_rkd_path(check_name)

    return [sys.executable, '-m', 'rkd', '--imports', module_name, task_name]


def rkd_module_exists(check_name: str) -> bool:
    module_name, task_name = split_rkd_path(check_name)

    try:
        tasks = SyntaxParsing.parse_imports_by_list_of_classes([module_name])

    except ParsingException:
        return False

    for task in tasks:
        task: TaskDeclarationInterface

        if task.get_task_to_execute().get_name() == task_name:
            return True

    return False


def add_rkd_environment_variables(env: dict, check_name: str) -> dict:
    env['RKD_UI'] = 'false'

    return env




import abc
import os
import re


class BaseLoadAverageCheck(abc.ABC):
    @staticmethod
    def get_complete_avg() -> str:
        avg = os.getloadavg()
        return '{:.2f}, {:.2f}, {:.2f}'.format(avg[0], avg[1], avg[2])

    @staticmethod
    def get_load_average(timing: str) -> float:
        if os.getenv('MOCK_LOAD_AVERAGE'):
            return float(os.getenv('MOCK_LOAD_AVERAGE'))

        if timing not in ['1', '5', '15']:
            raise Exception('Invalid argument, expected type to be: 1, 5 or 15')

        load = {'1': os.getloadavg()[0], '5': os.getloadavg()[1], '15': os.getloadavg()[2]}
        return load[timing]

    @staticmethod
    def available_cpu_count() -> int:
        """ Number of available virtual or physical CPUs on this system, i.e.
        user/real as output by time(1) when called with an optimally scaling
        userspace-only program

        :url: https://stackoverflow.com/a/1006301/6782994
        """

        if os.getenv('MOCK_CPU_COUNT'):
            return int(os.getenv('MOCK_CPU_COUNT'))

        # cpuset
        # cpuset may restrict the number of *available* processors
        try:
            with open('/proc/self/status') as f:
                m = re.search(r'(?m)^Cpus_allowed:\s*(.*)$', f.read())

                if m:
                    res = bin(int(m.group(1).replace(',', ''), 16)).count('1')
                    if res > 0:
                        return int(res)
        except IOError:
            pass

        # Python 2.6+
        try:
            import multiprocessing
            return multiprocessing.cpu_count()
        except (ImportError, NotImplementedError):
            pass

        # Linux
        try:
            with open('/proc/cpuinfo') as f:
                res = f.read().count('processor\t:')

                if res > 0:
                    return int(res)
        except IOError:
            pass

        raise Exception('Can not determine number of CPUs on this system')



"""
Scheduler
=========

Schedules execution of health checks verification every X seconds.
"""
import time
from threading import Thread
from traceback import format_exc
from .repository import Repository
from .runner import Runner
from rkd.api.inputoutput import IO


class Scheduler(object):
    runner: Runner
    thread: Thread
    repository: Repository
    io: IO

    def __init__(self, runner: Runner, repository: Repository, io: IO):
        self.runner = runner
        self.repository = repository
        self.io = io

    def schedule_jobs_in_background(self, every_seconds: int):
        self.thread = Thread(target=self._run_checks_infinitely, args=(self, every_seconds))
        self.thread.setDaemon(True)
        self.thread.start()

    def _run_checks_infinitely(self, scheduler, every_seconds: int):
        while True:
            try:
                configured_checks = self.repository.get_configured_checks(with_disabled=False)

                self.io.info('Running {} checks...'.format(len(configured_checks)))
                self.runner.run_checks(configured_checks)

            except Exception:
                self.io.error('Exception happened during processing')
                self.io.error(format_exc())

            self.io.debug('Sleeping {}s'.format(every_seconds))
            time.sleep(every_seconds)
